import { TemplateResult } from "lit";
import { Ref } from "lit-html/directives/ref.js";
export type FormBuilderConfig = {
    gap?: "small" | "medium" | "large";
    variant?: "fill" | "transparent";
    groupSeparator?: boolean;
    fieldSize?: "small" | "medium";
    groups: Record<string, FormBuilderGroup>;
};
export type FormBuilderGroup = {
    direction?: "vertical" | "horizontal";
    variant?: "normal" | "compact";
    isCollapsible?: boolean;
    isCollpased?: boolean;
    canDuplicate?: boolean;
    label?: FormBuilderLabel;
    fields: Record<string, FormBuilderField>;
    show?: FormBuilderShowCondition;
};
export type FormBuilderBaseField = {
    id?: string;
    state?: "default" | "success" | "error" | "warning";
    className?: string;
    dataQA?: string;
    label?: FormBuilderLabel;
    description?: string;
    canDuplicate?: boolean;
    validationRules?: FormBuilderValidationRules;
    show?: FormBuilderShowCondition;
};
export type FormBuilderTextInputField = FormBuilderBaseField & {
    type: "text";
    placeholder?: string;
    autoComplete?: boolean;
};
export type FormBuilderField = FormBuilderTextInputField;
export type FormBuilderShowCondition = (values: FormBuilderValues) => boolean;
export type FormBuilderLabel = {
    title: string;
    description?: string;
    icon?: string;
};
export type FormBuilderValidationRuleTriggers = "blur" | "keyup" | "click" | "keypress" | "focus" | "input" | "change";
export type FormBuilderValidationRule = {
    when?: FormBuilderValidationRuleTriggers[];
    message?: string;
};
export type FormBuilderValidationRequiredRule = FormBuilderValidationRule & {
    name: "required";
};
export type FormBuilderCustomValidationRule = FormBuilderValidationRule & {
    name: "custom";
    when?: FormBuilderValidationRuleTriggers[];
    validate: FormBuilderValidatorFunction;
};
export type FormBuilderValidatorFunction = (value: string | unknown[], params?: Record<string, unknown>) => boolean;
export type FormBuilderGenericValidationRule = FormBuilderValidationRequiredRule | FormBuilderCustomValidationRule;
export type FormBuilderValidationRules = FormBuilderGenericValidationRule[];
export type FormBuilderState = {
    isValid: boolean;
    isChanged: boolean;
    errors: Record<string, string>;
    refs: Record<string, Ref<HTMLInputElement>>;
    rules: Record<string, FormBuilderValidationRules | undefined>;
    errorRefs: Record<string, Ref<HTMLElement>>;
    showFunctions: Map<Ref<HTMLElement>, FormBuilderShowCondition>;
};
export type FormBuilderValues = Record<string, Record<string, string | string[] | number | number[]>>;
export type FormBuilderFieldRenderFunction = (name: string, field: FormBuilderField, idx: number, fieldRef: Ref<HTMLInputElement>, params?: Record<string, unknown>) => TemplateResult;
